@@ -33,14 +33,10 @@ minio - MinIO Python SDK for Amazon S3 Compatible Cloud Storage
 
 __title__ = "minio-py"
 __author__ = "MinIO, Inc."
-__version__ = "7.1.14"
+__version__ = "7.1.16"
 __license__ = "Apache 2.0"
 __copyright__ = "Copyright 2015, 2016, 2017, 2018, 2019, 2020 MinIO, Inc."
 
-import threading
-
-__LOCALE_LOCK__ = threading.Lock()
-
 # pylint: disable=unused-import
 from .api import Minio
 from .error import InvalidResponseError, S3Error, ServerError
